package com.github.jfcloud.gene.sample.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.jfcloud.gene.common.constant.WhetherEnum;
import com.github.jfcloud.gene.common.util.CustomIdGenerator;
import com.github.jfcloud.gene.file.service.FileInfoService;
import com.github.jfcloud.gene.sample.entity.*;
import com.github.jfcloud.gene.sample.mapper.InspectionCommissionMapper;
import com.github.jfcloud.gene.sample.mapper.SampleAnimalMapper;
import com.github.jfcloud.gene.sample.mapper.SampleCheckOrderMapper;
import com.github.jfcloud.gene.sample.mapper.SampleInfoMapper;
import com.github.jfcloud.gene.sample.service.CommissionDetailService;
import com.github.jfcloud.gene.sample.service.SampleAnimalService;
import com.github.jfcloud.gene.sample.service.SampleCheckItemService;
import com.github.jfcloud.gene.sample.service.SampleEditService;
import com.github.jfcloud.gene.sample.vo.*;
import com.github.jfcloud.gene.util.LocalWord;
import com.github.jfcloud.gene.util.WordDataService;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.lang.reflect.Field;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class SampleAnimalServiceImpl extends ServiceImpl<SampleAnimalMapper, SampleAnimal> implements SampleAnimalService, SampleEditService {

    private final SampleInfoMapper sampleInfoMapper;
    private final InspectionCommissionMapper inspectionCommissionMapper;
    private final SampleCheckOrderMapper checkOrderMapper;
    private final CommissionDetailService commissionDetailService;
    private final SampleCheckItemService sampleCheckItemService;
    private final WordDataService wordDataService;
    private final FileInfoService fileInfoService;

    @Override
    public void removeBySampleId(Long sampleId) {
        List<SampleAnimal> sampleAnimals = list(new LambdaQueryWrapper<>(SampleAnimal.class)
                .select(SampleAnimal::getId)
                .eq(SampleAnimal::getSampleId, sampleId));
        if (sampleAnimals.isEmpty()) {
            return;
        }

        Set<Long> sampleAnimalIdSet = sampleAnimals.stream().map(SampleAnimal::getId).collect(Collectors.toSet());
        removeBatchByIds(sampleAnimalIdSet);

        //删除委托单
        inspectionCommissionMapper.delete(new LambdaQueryWrapper<>(InspectionCommission.class).in(InspectionCommission::getSampleAnimalId, sampleAnimalIdSet));

        //删除送检申请单
        checkOrderMapper.delete(new LambdaQueryWrapper<>(SampleCheckOrder.class).in(SampleCheckOrder::getSampleAnimalId, sampleAnimalIdSet));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void save(Long sampleId, SampleSubmitVo vo) {
        Assert.notNull(vo.getAnimal(), "动物样本送检不能为空");

        //先删除之前的记录
        removeBySampleId(sampleId);

        //保存动物样本送检信息
        SampleAnimal sampleAnimal = BeanUtil.copyProperties(vo.getAnimal(), SampleAnimal.class);
        sampleAnimal.setId(CustomIdGenerator.nextId());
        sampleAnimal.setSampleId(sampleId);
        sampleAnimal.insert();

        //保存委托列表，通过JSON序列化转为列表
        CommissionObjVo commissionObj = vo.getAnimal().getCommission();
        JSONObject commissionJson = JSON.parseObject(JSON.toJSONString(commissionObj, SerializerFeature.WriteMapNullValue));
        for (String fieldName : commissionJson.keySet()) {
            JSONObject fieldJson = commissionJson.getJSONObject(fieldName);
            if (fieldJson == null) {
                continue;
            }
            InspectionCommission commission = JSON.parseObject(fieldJson.toJSONString(), InspectionCommission.class);
            commission.setDetectType(fieldName);
            commission.setId(CustomIdGenerator.nextId());
            commission.setSampleAnimalId(sampleAnimal.getId());
            commission.insert();

            //保存样本类型
            JSONArray array = fieldJson.getJSONArray("detailList");
            if (array.isEmpty()) {
                log.warn("{} 委托样本类型为空", fieldName);
                continue;
            }

            List<CommissionDetail> commissionDetails = JSON.parseArray(array.toJSONString(), CommissionDetail.class)
                    .stream()
                    .filter(item -> StrUtil.isNotBlank(item.getSampleType()))
                    .peek(item -> item.setInspectionCommissionId(commission.getId()))
                    .collect(Collectors.toList());
            commissionDetailService.saveBatch(commissionDetails);
        }

        //保存送检申请单
        CheckOrderObjVo orderObj = vo.getAnimal().getOrder();
        JSONObject orderJson = JSON.parseObject(JSON.toJSONString(orderObj, SerializerFeature.WriteMapNullValue));
        for (String fieldName : orderJson.keySet()) {
            JSONObject fieldJson = orderJson.getJSONObject(fieldName);
            if (fieldJson == null) {
                continue;
            }
            SampleCheckOrder order = JSON.parseObject(fieldJson.toJSONString(), SampleCheckOrder.class);
            order.setApplyType(fieldName);
            order.setId(CustomIdGenerator.nextId());
            order.setSampleAnimalId(sampleAnimal.getId());
            order.insert();

            //保存送检申请单条目
            JSONArray array = fieldJson.getJSONArray("items");
            if (array.isEmpty()) {
                log.warn("type={} orderCode={} 送检申请单条目为空", fieldName, order.getOrderCode());
                continue;
            }

            List<SampleCheckItem> sampleCheckItems = JSON.parseArray(array.toJSONString(), SampleCheckItem.class);
            sampleCheckItems.forEach(item -> item.setCheckOrderId(order.getId()));
            sampleCheckItemService.saveBatch(sampleCheckItems);
        }


        log.info("保存动物样本送检信息成功");
//        generate(sampleId);
    }

    @SneakyThrows
    @Override
    public void generate(Long sampleId) {
        SampleInfo sampleInfo = sampleInfoMapper.selectById(sampleId);
        SampleAnimalVo sampleAnimal = getDetail(sampleId);

        //文件名称：项目名称-申请人-申请日期-随机四位.docx
        String applyDate = DateUtil.format(sampleInfo.getApplyTime(), DatePattern.CHINESE_DATE_PATTERN);
        String fileName = String.format("动物样本检测送检申请表-%s-%s-%s-%s.docx",
                sampleInfo.getProjectName(), sampleInfo.getApplicantName(), applyDate,
                RandomUtil.randomString(RandomUtil.BASE_CHAR_NUMBER_LOWER, 4));
        log.info("生成word 项目名称：{} 文件名称：{}", sampleInfo.getProjectName(), fileName);

        JSONObject dataMap = new JSONObject();
        wordDataService.fillDocCreator(dataMap);
        JSONObject sampleInfoObj = JSON.parseObject(JSON.toJSONStringWithDateFormat(sampleInfo, DatePattern.NORM_DATETIME_PATTERN));
        dataMap.put("sampleInfo", sampleInfoObj);
        JSONObject sampleAnimalObj = JSON.parseObject(JSON.toJSONStringWithDateFormat(sampleAnimal, DatePattern.NORM_DATETIME_PATTERN));
        dataMap.put("sampleAnimal", sampleAnimalObj);

        //检测人员
        if (StrUtil.isNotEmpty(sampleInfo.getInspector())) {
            String inspectorName = JSON.parseArray(sampleInfo.getInspector())
                    .stream()
                    .map(item -> {
                        JSONObject obj = (JSONObject) item;
                        return obj.getString("name");
                    })
                    .collect(Collectors.joining(" "));
            sampleInfoObj.put("inspectorName", inspectorName);
        }

        //

        String modelName = "animalSampleTesting.ftl";
        if (LocalWord.exportWord(modelName, dataMap, fileName)) {
            return;
        }

        fileInfoService.uploadFileWithFileName(modelName, fileName, sampleInfo.getId(), sampleInfo.getStatus(), dataMap);
    }

    @Override
    public SampleAnimalVo getDetail(Long sampleId) {
        SampleAnimal sampleAnimal = getOne(new LambdaQueryWrapper<>(SampleAnimal.class)
                .eq(SampleAnimal::getSampleId, sampleId)
                .eq(SampleAnimal::getDeleted, WhetherEnum.NO.getCode()));
        if (sampleAnimal == null) {
            return null;
        }

        SampleAnimalVo sampleAnimalVo = BeanUtil.copyProperties(sampleAnimal, SampleAnimalVo.class);

        //委托列表，列表转为对象，null值转为空对象
        List<CommissionVo> commissionList = inspectionCommissionMapper.getCommissionList(sampleAnimal.getId());
        CommissionObjVo commissionObj = new CommissionObjVo();
        Field[] fields = ReflectUtil.getFields(CommissionObjVo.class);
        for (Field field : fields) {
            String fieldName = field.getName();
            CommissionVo commissionVo = commissionList.stream()
                    .filter(item -> fieldName.equals(item.getDetectType()))
                    .findFirst()
                    .orElse(new CommissionVo());
            ReflectUtil.setFieldValue(commissionObj, fieldName, commissionVo);
        }
        sampleAnimalVo.setCommission(commissionObj);

        //送检申请单，列表转为对象，null值转为空对象
        List<SampleCheckOrderVo> orderList = checkOrderMapper.getOrderList(sampleAnimal.getId());
        CheckOrderObjVo orderObj = new CheckOrderObjVo();
        Field[] orderFields = ReflectUtil.getFields(CheckOrderObjVo.class);
        for (Field field : orderFields) {
            String fieldName = field.getName();
            SampleCheckOrderVo orderVo = orderList.stream()
                    .filter(item -> fieldName.equals(item.getApplyType()))
                    .findFirst()
                    .orElse(new SampleCheckOrderVo());
            ReflectUtil.setFieldValue(orderObj, fieldName, orderVo);
        }
        sampleAnimalVo.setOrder(orderObj);

        return sampleAnimalVo;
    }
}
